$Env:CONDA_EXE = "/opt/concourse/worker/volumes/live/15a822e7-77c7-4653-65e4-9ce50f6b081b/volume/conda_1617142310313/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold/bin/conda"
$Env:_CE_M = ""
$Env:_CE_CONDA = ""
$Env:_CONDA_ROOT = "/opt/concourse/worker/volumes/live/15a822e7-77c7-4653-65e4-9ce50f6b081b/volume/conda_1617142310313/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold"
$Env:_CONDA_EXE = "/opt/concourse/worker/volumes/live/15a822e7-77c7-4653-65e4-9ce50f6b081b/volume/conda_1617142310313/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold/bin/conda"

Import-Module "$Env:_CONDA_ROOT\shell\condabin\Conda.psm1"
Add-CondaEnvironmentToPrompt